@@ -33,6 +33,7 @@
 #include <errno.h>
 #include <signal.h>
 #include <ctype.h>
+#include <gnome.h>
 
 #include "client.h"
 #include "tetrinet.h"
@@ -44,15 +45,13 @@
 #define PORT 31457
 #define SPECPORT 31458
 
-int fdin[2], fdout[2]; /* two pipes, in and out */
-
-int inputcbid;
-
 int connected;
 char server[128];
-int clientpid;
 
-static int sock, connecterror;
+static int sock;
+static GIOChannel *io_channel;
+static guint source;
+static int resolved;
 
 /* structures and arrays for message translation */
 
@@ -97,6 +96,9 @@ struct inmsgt inmsgtable[] = {
     {IN_SPECLIST, "speclist"},
     {IN_SMSG, "smsg"},
     {IN_SACT, "sact"},
+
+    {IN_BTRIXNEWGAME, "btrixnewgame"},
+
     {0, 0}
 };
 
@@ -137,98 +139,43 @@ struct outmsgt outmsgtable[] = {
     {OUT_GMSG, "gmsg"},
 
     {OUT_VERSION, "version"},
+
+    {OUT_CLIENTINFO, "clientinfo"},
+
     {0, 0}
 };
 
-static void client_inputfunc (void);
-
 /* functions which set up the connection */
 static void client_process (void);
-static void client_cleanpipe (void);
-static int client_mainloop (void);
-static int client_connect (void);
+static gpointer client_resolv_hostname (void);
 static void client_connected (void);
-static int client_disconnect (void);
 
 /* some other useful functions */
+static gboolean io_channel_cb (GIOChannel *source, GIOCondition condition);
 static int client_sendmsg (char *str);
-static int client_readmsg (char *str, int len);
+static int client_readmsg (gchar **str);
 static void server_ip (unsigned char buf[4]);
 
 enum inmsg_type inmsg_translate (char *str);
 char *outmsg_translate (enum outmsg_type);
 
-void client_initpipes (void)
-{
-    pipe (fdin);
-    pipe (fdout);
-
-    inputcbid = gdk_input_add (fdin[0], GDK_INPUT_READ,
-                               (GdkInputFunction)client_inputfunc, NULL);
-}
-
-void client_destroypipes (void)
-{
-}
-
 void client_init (const char *s, const char *n)
 {
     int i;
     GTET_O_STRCPY(server, s);
     GTET_O_STRCPY(nick, n);
 
+    connectingdialog_new ();
+
     /* wipe spaces off the nick */
     for (i = 0; nick[i]; i ++)
-        if (isspace (nick[i])) nick[i] = 0;
-
-    if (clientpid) {
-        if (connected) client_destroy ();
-        else client_connectcancel ();
-    }
-
-    connectingdialog_new ();
+      if (isspace (nick[i]))
+        nick[i] = 0;
 
     /* set the game mode */
     inmsg_change();
-
-    if ((clientpid = fork()) == 0) {
-        client_process ();
-        _exit (0);
-    }
-}
-
-void client_connectcancel (void)
-{
-    /* just kill the process */
-    if (clientpid) kill (clientpid, SIGTERM);
-    clientpid = 0;
-}
-
-void client_destroy (void)
-{
-    /* tell it to disconnect, then wait for it to die */
-    client_outmessage (OUT_DISCONNECT, NULL);
-    waitpid (clientpid, NULL, 0);
-    clientpid = 0;
-}
-
-/* a function that reads stuff from the pipe */
-static void client_inputfunc (void)
-{
-    char buf[1024];
-    char *token;
-    enum inmsg_type msgtype;
-
-    fdreadline (fdin[0], buf);
-
-    /* split the message */
-    token = strtok (buf, " ");
-    if (token == NULL) return;
-    msgtype = inmsg_translate (token);
-    token = strtok (NULL, "");
-
-    /* process it */
-    tetrinet_inmessage (msgtype, token);
+    
+    client_process ();
 }
 
 void client_outmessage (enum outmsg_type msgtype, char *str)
@@ -239,107 +186,110 @@ void client_outmessage (enum outmsg_type msgtype, char *str)
         GTET_O_STRCAT(buf, " ");
         GTET_O_STRCAT(buf, str);
     }
-    write (fdout[1], buf, strlen(buf));
-    write (fdout[1], "\n", 1);
+    switch (msgtype)
+    {
+      case OUT_DISCONNECT : client_disconnect (); break;
+      case OUT_CONNECTED : client_connected (); break;
+      default : client_sendmsg (buf);
+    }
 }
 
 void client_inmessage (char *str)
 {
-    write (fdin[1], str, strlen(str));
-    write (fdin[1], "\n", 1);
+    enum inmsg_type msgtype;
+    gchar **tokens, *final;
+
+    /* split the message */
+    tokens = g_strsplit (str, " ", 256);
+    msgtype = inmsg_translate (tokens[0]);
+
+    /* process it */
+    final = g_strjoinv (" ", &tokens[1]);
+    tetrinet_inmessage (msgtype, final);
+    g_strfreev (tokens);
+    g_free (final);
 }
 
 /* these functions set up the connection */
 
 void client_process (void)
 {
-    errno = 0;
-    if (client_connect () == -1) {
-        char errmsg[1024];
+  GString *s1 = g_string_sized_new(80);
+  GString *s2 = g_string_sized_new(80);
+  unsigned char ip[4];
+  GString *iphashbuf = g_string_sized_new(11);
+  unsigned int i, len;
+  int l;
+  GThread *thread;
+        
+  errno = 0;
+  resolved = 0;
+  
+  thread = g_thread_create ((GThreadFunc) client_resolv_hostname, NULL, FALSE, NULL);
+  
+  /* wait until the hostname is resolved */
+  while (resolved == 0)
+  {
+    if (gtk_events_pending ())
+      gtk_main_iteration ();
+  }
 
-        GTET_O_STRCPY(errmsg, "noconnecting ");
+  if (resolved == -1) {
+    char errmsg[1024];
 
-        if (errno)        GTET_O_STRCAT(errmsg, strerror (errno));
-        else if (h_errno) GTET_O_STRCAT(errmsg, "Unknown host");
+    GTET_O_STRCPY(errmsg, "noconnecting ");
 
-        client_inmessage (errmsg);
+    if (errno)        GTET_O_STRCAT(errmsg, strerror (errno));
+    else if (h_errno) GTET_O_STRCAT(errmsg, _("Couldn't resolve hostname."));
 
-        return;
-    }
-    connecterror = 0;
-    client_cleanpipe ();
-    client_mainloop ();
-    client_disconnect ();
+    client_inmessage (errmsg);
+    
     return;
+ }
+
+  /**
+   * Set up the g_io_channel
+   * We should set it with no encoding and no buffering, just to simplify things */
+  io_channel = g_io_channel_unix_new (sock);
+  g_io_channel_set_encoding (io_channel, NULL, NULL);
+  g_io_channel_set_buffered (io_channel, FALSE);
+  source = g_io_add_watch (io_channel, G_IO_IN, (GIOFunc)io_channel_cb, NULL);
+
+  /* construct message */
+  if (gamemode == TETRIFAST)
+    g_string_printf (s1, "tetrifaster %s 1.13", nick);
+  else
+    g_string_printf (s1, "tetrisstart %s 1.13", nick);
+
+  /* do that encoding thingy */
+  server_ip (ip);
+  g_string_printf (iphashbuf, "%d",
+                   ip[0]*54 + ip[1]*41 + ip[2]*29 + ip[3]*17);
+  l = iphashbuf->len;
+
+  g_string_append_c(s2, 0);
+  for (i = 0; s1->str[i]; i ++)
+    g_string_append_c(s2, ((((s2->str[i] & 0xFF) +
+                     (s1->str[i] & 0xFF)) % 255) ^
+                      iphashbuf->str[i % l]));
+  g_assert(s1->len == i);
+  g_assert(s2->len == (i + 1));
+  len = i + 1;
+
+  g_string_truncate(s1, 0);
+  for (i = 0; i < len; i ++)
+    g_string_append_printf(s1, "%02X", s2->str[i] & 0xFF);
+
+  /* now send to server */
+  client_sendmsg (s1->str);
+
+  g_string_free(s1, TRUE);
+  g_string_free(s2, TRUE);
+  g_string_free(iphashbuf, TRUE);
 }
 
-void client_cleanpipe ()
-{
-    fd_set rfds;
-    struct timeval tv;
-    char buf[1024];
 
-    while (1) {
-        FD_ZERO (&rfds);
-        FD_SET (fdout[0], &rfds);
-        tv.tv_sec = 0; tv.tv_usec = 0;
-        if (select (fdout[0]+1, &rfds, NULL, NULL, &tv)) fdreadline (fdout[0], buf);
-        else return;
-    }
-}
-
-int client_mainloop (void)
-{
-    fd_set rfds;
-    struct timeval tv;
-    int m;
-
-    /* read in stuff from the socket and the pipe */
-    while (1) {
-        FD_ZERO (&rfds);
-        FD_SET (sock, &rfds);
-        FD_SET (fdout[0], &rfds);
-        m = sock > fdout[0] ? sock : fdout[0];
-        tv.tv_sec = 0;
-        tv.tv_usec = 50000; /* wait a little while */
-        if (select (m+1, &rfds, NULL, NULL, &tv))
-        {
-            char buf[1024]; /* a big buffer */
-
-            /* we have something to read */
-
-            /* is it the socket?? */
-            FD_ZERO (&rfds); FD_SET (sock, &rfds);
-            tv.tv_sec = 0; tv.tv_usec = 0;
-            if (select (sock+1, &rfds, NULL, NULL, &tv)) {
-                if (client_readmsg (buf, sizeof(buf)) < 0)
-                    return -1;
-                client_inmessage (buf); /* send to parent process */
-                if (strncmp ("noconnecting", buf, 12) == 0) {
-                    connecterror = 1;
-                    goto clientend;
-                }
-            }
-
-            /* or it the pipe?? */
-            FD_ZERO (&rfds); FD_SET (fdout[0], &rfds);
-            tv.tv_sec = 0; tv.tv_usec = 0;
-            if (select (fdout[0]+1, &rfds, NULL, NULL, &tv)) {
-                fdreadline (fdout[0], buf);
-                if (strcmp (buf, outmsg_translate(OUT_DISCONNECT)) == 0)
-                    goto clientend;
-                else if (strcmp (buf, outmsg_translate(OUT_CONNECTED)) == 0)
-                    client_connected ();
-                else
-                    client_sendmsg (buf);
-            }
-        }
-    }
-clientend:
-    return 0;
-}
-
-int client_connect (void)
+gpointer client_resolv_hostname (void)
 {
 #ifdef USE_IPV6
     char hbuf[NI_MAXHOST];
@@ -361,7 +311,8 @@ int client_connect (void)
     if (getaddrinfo(server, service, &hints, &res0)) {
         /* set errno = 0 so that we know it's a getaddrinfo error */
         errno = 0;
-        return -1;
+        resolved = -1;
+        g_thread_exit (GINT_TO_POINTER (-1));
     }
     for (res = res0; res; res = res->ai_next) {
         sock = socket (res->ai_family, res->ai_socktype, res->ai_protocol);
@@ -370,7 +321,8 @@ int client_connect (void)
                 continue;
             else {
                 freeaddrinfo(res0);
-                return -1;
+                resolved = -1;
+                g_thread_exit (GINT_TO_POINTER (-1));
             }
         }
         getnameinfo(res->ai_addr, res->ai_addrlen, hbuf, sizeof(hbuf), NULL, 0, 0);
@@ -381,7 +333,8 @@ int client_connect (void)
             } else {
                 close(sock);
                 freeaddrinfo(res0);
-                return -1;
+                resolved = -1;
+                g_thread_exit (GINT_TO_POINTER (-1));
             }
         }
         break;
@@ -392,7 +345,8 @@ int client_connect (void)
     if (h == 0) {
         /* set errno = 0 so that we know it's a gethostbyname error */
         errno = 0;
-        return -1;
+        resolved = -1;
+        g_thread_exit (GINT_TO_POINTER (-1));
     }
     memset (&sa, 0, sizeof (sa));
     memcpy (&sa.sin_addr, h->h_addr, h->h_length);
@@ -400,54 +354,18 @@ int client_connect (void)
     sa.sin_port = htons (spectating?SPECPORT:PORT);
 
     sock = socket (sa.sin_family, SOCK_STREAM, 0);
-    if (sock < 0) return -1;
+    if (sock < 0)
+        g_thread_exit (GINT_TO_POINTER (-1));
 
     if (connect (sock, (struct sockaddr *)&sa, sizeof(sa)) < 0)
-        return -1;
+    {
+        resolved = -1;
+        g_thread_exit (GINT_TO_POINTER (-1));
+    }
 #endif
 
-    /* say hello to the server */
-    {
-        GString *s1 = g_string_sized_new(80);
-        GString *s2 = g_string_sized_new(80);
-        unsigned char ip[4];
-        GString *iphashbuf = g_string_sized_new(11);
-        unsigned int i, len;
-        int l;
-        
-        /* construct message */
-        if (gamemode == TETRIFAST)
-          g_string_printf (s1, "tetrifaster %s 1.13", nick);
-        else
-          g_string_printf (s1, "tetrisstart %s 1.13", nick);
-
-        /* do that encoding thingy */
-        server_ip (ip);
-        g_string_printf (iphashbuf, "%d",
-                          ip[0]*54 + ip[1]*41 + ip[2]*29 + ip[3]*17);
-        l = iphashbuf->len;
-
-        g_string_append_c(s2, 0);
-        for (i = 0; s1->str[i]; i ++)
-          g_string_append_c(s2, ((((s2->str[i] & 0xFF) +
-                                   (s1->str[i] & 0xFF)) % 255) ^
-                                 iphashbuf->str[i % l]));
-        g_assert(s1->len == i);
-        g_assert(s2->len == (i + 1));
-        len = i + 1;
-
-        g_string_truncate(s1, 0);
-        for (i = 0; i < len; i ++)
-          g_string_append_printf(s1, "%02X", s2->str[i] & 0xFF);
-
-        /* now send to server */
-        client_sendmsg (s1->str);
-
-        g_string_free(s1, TRUE);
-        g_string_free(s2, TRUE);
-        g_string_free(iphashbuf, TRUE);
-    }
-    return 0;
+    resolved = 1;
+    return (GINT_TO_POINTER (1));
 }
 
 void client_connected (void)
@@ -456,50 +374,122 @@ void client_connected (void)
     client_inmessage ("connect");
 }
 
-int client_disconnect (void)
+void client_disconnect (void)
 {
-    shutdown (sock, 2);
-    close (sock);
+    if (connected)
+    {
+      if (gtk_main_level())
+        client_inmessage ("disconnect");
+      g_source_destroy (g_main_context_find_source_by_id (NULL, source));
+      g_io_channel_shutdown (io_channel, TRUE, NULL);
+      g_io_channel_unref (io_channel);
+      shutdown (sock, 2);
+      close (sock);
+      connected = 0;
 
-    if (!connecterror || connected) client_inmessage ("disconnect");
-
-    return 0;
+      // Allow for sending the blocktrix init on reconnect.
+      pnumrec = 0;
+    }
 }
 
 
 /* some other useful functions */
 
+static gboolean
+io_channel_cb (GIOChannel *source, GIOCondition condition)
+{
+  gchar *buf;
+  
+  source = source; /* get rid of the warnings */
+  
+  switch (condition)
+  {
+    case G_IO_IN :
+    {
+      if (client_readmsg (&buf) < 0)
+      {
+        g_warning ("client_readmsg failed, aborting connection\n");
+        client_disconnect ();
+      }
+      else
+      {
+        if (strlen (buf)) client_inmessage (buf);
+        
+        if (strncmp ("noconnecting", buf, 12) == 0)
+        {
+          connected = 1; /* so we can disconnect :) */
+          client_disconnect ();
+        }
+        g_free (buf);
+      }
+    }; break;
+    default : break;
+  }
+  
+  return TRUE;
+}
+
 int client_sendmsg (char *str)
 {
     char c = 0xFF;
+    GError *error = NULL;
     
+    g_io_channel_write_chars (io_channel, str, -1, NULL, &error);
+    g_io_channel_write_chars (io_channel, &c, 1, NULL, &error);
+    g_io_channel_flush (io_channel, &error);
+
 #ifdef DEBUG
     printf ("> %s\n", str);
 #endif
 
-    if (write (sock, str, strlen (str))==-1) return -1;
-    if (write (sock, &c, 1)==-1) return -1;
-
     return 0;
 }
 
-int client_readmsg (char *str, int len)
+int client_readmsg (gchar **str)
 {
-    int i = 0;
-    /* read it in one char at a time */
-    for (;i < len-1; i++) {
-        if (read (sock, &str[i], 1) != 1)
-            /* we have an error */
+    gint bytes = 0;
+    gchar buf[1024];
+    GError *error = NULL;
+    gint i = 0;
+  
+    do
+    {
+      switch (g_io_channel_read_chars (io_channel, &buf[i], 1, &bytes, &error))
+      {
+        case G_IO_STATUS_EOF :
+          g_warning ("End of file (server closed connection).");
+          return -1;
+          break;
+        
+        case G_IO_STATUS_AGAIN :
+          g_warning ("Resource temporarily unavailable.");
+          return -1;
+          break;
+        
+        case G_IO_STATUS_ERROR :
+          g_warning ("Error");
+          return -1;
+          break;
+        
+        case G_IO_STATUS_NORMAL :
+          if (error != NULL)
+          {
+            g_warning ("ERROR READING: %s\n", error->message);
+            g_error_free (error);
             return -1;
-        if (str[i]==(char)0xFF) break;
-    }
-    str[i] = 0;
+          }; break;
+      }
+      i++;
+    } while ((bytes == 1) && (buf[i-1] != (gchar)0xFF) && (i<1024));
+    buf[i-1] = 0;
 
 #ifdef DEBUG
-    printf ("< %s\n", str);
+    printf ("< %s\n", buf);
 #endif
+    
+    *str = g_strdup (buf);
 
-    return i;
+    return 0;
 }
 
 void server_ip (unsigned char buf[4])
