@@ -34,12 +34,13 @@
 #include "commands.h"
 
 int timestampsenable;
+gboolean list_enabled;
 
 /* widgets that we have to do stuff with */
 static GtkWidget *playerlist, *textbox, *entrybox,
     *namelabel, *teamlabel, *infolabel, *textboxscroll, 
     *playerlist_scroll, *playerlist_vpaned, *channel_box,
-    *playerlist_channel_scroll, *label;
+    *playerlist_channel_scroll, *label, *channel_list;
 
 /* some more widgets for layout */
 static GtkWidget *table, *leftbox, *rightbox;
@@ -58,10 +59,11 @@ void channel_activated (GtkTreeView *treeview);
 
 GtkWidget *partyline_page_new (void)
 {
-    GtkWidget *widget, *box, *box1, *box2; /* generic temp variables */
+    GtkWidget *widget, *box, *box2; /* generic temp variables */
     GtkListStore *playerlist_model = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
     GtkCellRenderer *renderer = gtk_cell_renderer_text_new ();
     GList *focus_list = NULL;
+    gchar* aux;
 
     playerlist_channels = gtk_list_store_new (5, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
     work_model = gtk_list_store_new (5, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
@@ -70,7 +72,7 @@ GtkWidget *partyline_page_new (void)
     leftbox = gtk_vbox_new (FALSE, 4);
     /* chat thingy */
     /* channel list */
-    box1 = gtk_vbox_new (FALSE, 0);
+    channel_list = gtk_vbox_new (FALSE, 0);
     channel_box = GTK_WIDGET (gtk_tree_view_new_with_model (GTK_TREE_MODEL (playerlist_channels)));
     gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (channel_box), -1, _("Name"), renderer,
                                                  "text", 1, NULL);
@@ -89,9 +91,12 @@ GtkWidget *partyline_page_new (void)
     gtk_container_add (GTK_CONTAINER(playerlist_channel_scroll), channel_box);
     gtk_widget_set_size_request (playerlist_channel_scroll, -1, 100);
     label = gtk_label_new (NULL);
-    gtk_label_set_markup (GTK_LABEL (label), "<b>Channel List</b>");
-    gtk_box_pack_start (GTK_BOX (box1), label, FALSE, FALSE, 0);
-    gtk_box_pack_start (GTK_BOX (box1), playerlist_channel_scroll, TRUE, TRUE, 0);
+
+    aux = g_strconcat ("<b>", _("Channel List"), "</b>", NULL);
+    gtk_label_set_markup (GTK_LABEL (label), aux);
+    g_free (aux);
+    gtk_box_pack_start (GTK_BOX (channel_list), label, FALSE, FALSE, 0);
+    gtk_box_pack_start (GTK_BOX (channel_list), playerlist_channel_scroll, TRUE, TRUE, 0);
 
     /* textbox with scrollbars */
     box2 = gtk_vbox_new (FALSE, 0);
@@ -112,7 +117,7 @@ GtkWidget *partyline_page_new (void)
     /* vpaned widget */
     playerlist_vpaned = gtk_vpaned_new ();
     
-    gtk_paned_add1 (GTK_PANED (playerlist_vpaned), box1);
+    gtk_paned_add1 (GTK_PANED (playerlist_vpaned), channel_list);
     gtk_paned_add2 (GTK_PANED (playerlist_vpaned), box2);
     gtk_box_pack_start (GTK_BOX(leftbox), playerlist_vpaned, TRUE, TRUE, 0);
     
@@ -228,7 +233,7 @@ void partyline_status (char *status)
     gtk_label_set_text (GTK_LABEL(infolabel), status);
 }
 
-void partyline_text (char *text)
+void partyline_text (const gchar *text)
 {
     if (timestampsenable) {
         time_t now;
@@ -324,7 +329,7 @@ void textentry (GtkWidget *widget)
 
     if (strlen(text) == 0) return;
 
-    if (!strncmp("/list", text, strlen("/list")))
+    if (g_str_has_prefix(text, "/list"))
       stop_list(); /* Parsing can't be perfect,
                       so make sure they can do it by hand... */
     
@@ -332,8 +337,13 @@ void textentry (GtkWidget *widget)
     iso_text = g_locale_from_utf8 (text, -1, NULL, NULL, NULL);
 
 	/* FIXME : if there is an error while converting from UTF8 to current locale, we ignore the message */
-	if (iso_text == NULL) return;
+    if (iso_text == NULL)
+      return;
 
+    // Show the command if it's a /msg
+    if (g_str_has_prefix (text, "/msg"))
+      partyline_text (text);
+    
     tetrinet_playerline (iso_text);
     GTET_O_STRCPY (plhistory[plh_end], iso_text);
     gtk_entry_set_text (GTK_ENTRY(widget), "");
@@ -348,16 +358,62 @@ void textentry (GtkWidget *widget)
     g_free (iso_text);
 }
 
+static gboolean is_nick (GtkTreeModel *model,
+                         GtkTreePath *path,
+                         GtkTreeIter *iter,
+                         gpointer data)
+{
+  gchar *nick, *aux, *down;
+
+  gtk_tree_model_get (model, iter, 1, &nick, -1);
+  down = g_utf8_strdown (nick, -1);
+  path = path;
+  
+  if (g_str_has_prefix (down, data))
+  {
+    aux = g_strconcat (nick, ": ", NULL);
+    gtk_entry_set_text (GTK_ENTRY (entrybox), aux);
+    gtk_editable_set_position (GTK_EDITABLE (entrybox), -1);
+    
+    g_free (aux);
+    g_free (nick);
+    g_free (down);
+    return TRUE;
+  }
+  else
+  {
+    g_free (nick);
+    g_free (down);
+    return FALSE;
+  }
+}
+
+static void playerlist_complete_nick (void)
+{
+  GtkListStore *playerlist_model = GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (playerlist)));
+  gchar *text;
+  
+  text = g_utf8_strdown (gtk_entry_get_text (GTK_ENTRY (entrybox)), -1);
+  if (text == NULL) return;
+
+  gtk_tree_model_foreach (GTK_TREE_MODEL (playerlist_model), is_nick, text);
+
+  g_free (text);
+}
+
 static gint entrykey (GtkWidget *widget, GdkEventKey *key)
 {
     int keyval = key->keyval;
-    gchar *text;
+    gchar *text = NULL;
 
     if (keyval == GDK_Up || keyval == GDK_Down) {
         if (plh_cur == plh_end) {
             text = g_locale_from_utf8 (gtk_entry_get_text (GTK_ENTRY(widget)), -1, NULL, NULL, NULL);
-            GTET_O_STRCPY (plhistory[plh_end], text);
-            g_free (text);
+            if (text != NULL)
+            {
+              GTET_O_STRCPY (plhistory[plh_end], text);
+              g_free (text);
+            }
         }
         switch (keyval) {
         case GDK_Up:
@@ -385,6 +441,11 @@ static gint entrykey (GtkWidget *widget, GdkEventKey *key)
     else if (keyval == GDK_Left || keyval == GDK_Right) {
         return FALSE;
     }
+    else if (keyval == GDK_Tab)
+    {
+      playerlist_complete_nick ();
+      return TRUE;
+    }
     else {
         plh_cur = plh_end;
         return FALSE;
@@ -407,42 +468,78 @@ void partyline_add_channel (gchar *line)
   
   while ((g_scanner_get_next_token (scan) != G_TOKEN_INT) && !g_scanner_eof (scan));
   num = scan->value.v_int;
-  
-  while ((g_scanner_get_next_token (scan) != G_TOKEN_COMMENT_SINGLE) && !g_scanner_eof (scan));
-  utf8 = g_locale_to_utf8 (scan->value.v_comment, -1, NULL, NULL, NULL);
-  name = g_strconcat ("#", utf8, NULL);
-  
-  while ((g_scanner_get_next_token (scan) != G_TOKEN_IDENTIFIER) && !g_scanner_eof (scan));
-  players = g_strdup (scan->value.v_identifier);
 
-  if (strncmp (players, "FULL", 4))
+  g_scanner_get_next_token (scan); /* dump the ')' */
+  
+  if (g_scanner_peek_next_token (scan) == G_TOKEN_LEFT_BRACE)
   {
+    scan->config->cpair_comment_single = "# ";
+    
     while ((g_scanner_get_next_token (scan) != G_TOKEN_INT) && !g_scanner_eof (scan));
     actual = scan->value.v_int;
-
+    
     while ((g_scanner_get_next_token (scan) != G_TOKEN_INT) && !g_scanner_eof (scan));
     max = scan->value.v_int;
 
-    g_snprintf (final, 1024, "%d/%d %s", actual, max, players);
-  }
-  else
-    g_snprintf (final, 1024, "6/6 %s", players);
-  
-  g_scanner_get_next_token (scan); /* dump the ')' */
-  
-  if (g_scanner_get_next_token (scan) == G_TOKEN_LEFT_CURLY)
-  {
-    g_scanner_get_next_token (scan);
-    state = g_strdup (scan->value.v_identifier);
-  }
-  else
-    state = g_strdup ("IDLE");
-  
-  while ((g_scanner_get_next_token (scan) != G_TOKEN_RIGHT_PAREN) && !g_scanner_eof (scan));
-  if (line[scan->position] != 0)
-    desc = g_strstrip (g_locale_to_utf8 (&line[scan->position], -1, NULL, NULL, NULL));
-  else
+    while ((g_scanner_get_next_token (scan) != G_TOKEN_COMMENT_SINGLE) && !g_scanner_eof (scan));
+    utf8 = g_locale_to_utf8 (scan->value.v_comment, -1, NULL, NULL, NULL);
+    name = g_strconcat ("#", utf8, NULL);
+    
+    g_snprintf (final, 1024, "%d/%d", actual, max);
+
+    scan->config->cpair_comment_single = "{}";
+    while ((g_scanner_get_next_token (scan) != G_TOKEN_COMMENT_SINGLE) && !g_scanner_eof (scan));
+    if (!g_scanner_eof (scan))
+      state = g_strdup (scan->value.v_comment);
+    else
+      state = g_strdup ("IDLE");
+
     desc = g_strdup ("");
+    players = g_strdup ("");
+  }
+  else
+  {
+    while ((g_scanner_get_next_token (scan) != G_TOKEN_COMMENT_SINGLE) && !g_scanner_eof (scan));
+    utf8 = g_locale_to_utf8 (scan->value.v_comment, -1, NULL, NULL, NULL);
+    name = g_strconcat ("#", utf8, NULL);
+  
+    while ((g_scanner_get_next_token (scan) != G_TOKEN_IDENTIFIER) && !g_scanner_eof (scan));
+    players = g_strdup (scan->value.v_identifier);
+
+    if (players != NULL)
+    {
+      if (strncmp (players, "FULL", 4))
+      {
+        while ((g_scanner_get_next_token (scan) != G_TOKEN_INT) && !g_scanner_eof (scan));
+        actual = scan->value.v_int;
+
+        while ((g_scanner_get_next_token (scan) != G_TOKEN_INT) && !g_scanner_eof (scan));
+        max = scan->value.v_int;
+
+        g_snprintf (final, 1024, "%d/%d %s", actual, max, players);
+      }
+      else
+        g_snprintf (final, 1024, "%s", players);
+    }
+    else
+      g_snprintf (final, 1024, "UNK");
+  
+    g_scanner_get_next_token (scan); /* dump the ')' */
+  
+    if (g_scanner_get_next_token (scan) == G_TOKEN_LEFT_CURLY)
+    {
+      g_scanner_get_next_token (scan);
+      state = g_strdup (scan->value.v_identifier);
+    }
+    else
+      state = g_strdup ("IDLE");
+  
+    while ((g_scanner_get_next_token (scan) != G_TOKEN_RIGHT_PAREN) && !g_scanner_eof (scan));
+    if (line[scan->position] != 0)
+      desc = g_strstrip (g_locale_to_utf8 (&line[scan->position], -1, NULL, NULL, NULL));
+    else
+      desc = g_strdup ("");
+  }
   
   
   gtk_list_store_append (work_model, &iter);
@@ -512,7 +609,7 @@ gboolean partyline_update_channel_list (void)
   gchar cad[1024];
   
   /* if there is another update in progress, just go away silently */
-  if (list_issued == 0)
+  if (connected && list_enabled && (list_issued == 0))
   {
     list_issued++;
     gtk_list_store_clear (work_model);
@@ -573,3 +670,19 @@ void partyline_joining_channel (const gchar *channel)
   
   g_free (final);
 }
+
+void partyline_show_channel_list (gboolean show)
+{
+  /*
+   * If this function is called with TRUE, it will show the channel list, otherwise
+   * it'll hide it.
+   */
+  list_enabled = show;
+  if (list_enabled)
+  {
+    gtk_widget_show (channel_list);
+    partyline_update_channel_list ();
+  }
+  else
+    gtk_widget_hide (channel_list);
+}
